# TODO: Handle these exceptions properly

class LoginFailure(Exception):
	pass

class QuestionError(Exception):
	pass

class UploadError(Exception):
	pass

class UserError(Exception):
	pass

class NonPublicFunctionError(Exception):
	pass